import os
import psutil
import logging
import requests
from middleware.version import SDK_VERSION
from middleware.config import config

green_color = "\033[92m"
yellow_color = "\033[93m"
reset_color = "\033[0m"


class MwTracker:
    def __init__(self):

        self.project_name = config.project_name
        self.service_name = config.service_name
        self.access_token = config.access_token or ""

        self._health_check()
        if config.collect_metrics:
            self.collect_metrics()
        if config.collect_traces:
            self.collect_traces()
        if config.collect_logs:
            self.collect_logs()
        self._get_instrument_info()

    def collect_metrics(self):
        os.environ["OTEL_PYTHON_METER_PROVIDER"] = "sdk_meter_provider"
        from ._meter import collect_metrics

        collect_metrics()

    def collect_traces(self):
        os.environ["OTEL_PYTHON_TRACER_PROVIDER"] = "sdk_tracer_provider"
        from ._tracer import collect_traces

        collect_traces()

    def collect_logs(self):
        from ._logger import log_handler

        handler = log_handler()
        logging.getLogger().addHandler(handler)
        logging.setLogRecordFactory(self._set_custom_log_attr)

    def record_error(self, error):
        from ._tracer import record_error

        record_error(error)

    def django_instrument(self):
        if config.collect_traces:
            from opentelemetry.instrumentation.django import DjangoInstrumentor
            DjangoInstrumentor().instrument()

    def _set_custom_log_attr(self, *args, **kwargs):
        record = logging.LogRecord(*args, **kwargs)
        update_json = {"service.name": self.service_name, "mw.app.lang": "python"}
        if self.project_name:
            update_json["project.name"] = self.project_name
        record.__dict__.update(update_json)
        return record

    def _get_project_name(self, resource_attributes) -> str:
        project_name = None
        if resource_attributes is not None:
            attributes_list = resource_attributes.split(",")
            for attribute in attributes_list:
                key, value = attribute.split("=")
                if key == "project.name":
                    project_name = value

        return project_name

    def _health_check(self):
        if config.target == "" or ("https" not in config.target) :
            try:
                response = requests.get(
                    f"http://{config.mw_agent_service}:13133/healthcheck", timeout=5
                )
                if response.status_code != 200:
                    print(
                        f"[{yellow_color}WARN{reset_color}]: MW Agent Health Check is failing ...\nThis could be due to incorrect value of MW_AGENT_SERVICE\nIgnore the warning if you are using MW Agent older than 1.7.7 (You can confirm by running `mw-agent version`)"
                    )
            except requests.exceptions.RequestException as e:
                print(f"[{yellow_color}WARN{reset_color}]: MW Agent Health Check is failing ...\nException while MW Agent Health Check:{e}")

    def _get_instrument_info(self):
        if config.disable_info != True:
            print(f"{green_color}")
            print(
                f"Middleware Python Instrumentation Started\nSDK version: {SDK_VERSION}"
            )
            if config.target == "" or ("https" not in config.target):
                print(f"Using Agent Instrumentation")
            else:
                print(f"Using Serverless Instrumentation")
            print(f"OTLP Endpoint:{config.exporter_otlp_endpoint}")
            if config.collect_metrics:
                print(f"Metrics: Enabled")
            if config.collect_traces:
                print(f"Traces: Enabled")
            if config.collect_logs:
                print(f"Logs: Enabled")
            print(f"{reset_color}")



import os
import sys
import shutil
# from importlib.resources import path
import pkg_resources
import configparser
from middleware.version import SDK_VERSION, PYTHON_VERSION

CONFIG_FILE_PATH = os.getenv('MIDDLEWARE_CONFIG_FILE', os.path.join(os.getcwd(), "middleware.ini"))
DEFAULT_CONFIG_FILE_PATH = pkg_resources.resource_filename('middleware', 'middleware.ini')
# DEFAULT_CONFIG_FILE_PATH = path('middleware','middleware.ini')

class Config:
    def get_config(self, section, key, default):
        # Allowing OTEL level overrides at top priority
        if key in self.otel_config_binding:
            otel_env_value = os.environ.get(self.otel_config_binding[key], None)
            if otel_env_value is not None and otel_env_value != "":
                return otel_env_value

        # Allowing MW level ENV overrides
        if key in self.config_binding:
            env_value = os.environ.get(self.config_binding[key], None)
            if env_value is not None and env_value != "":
                return env_value

        return self.config.get(section, key, fallback=default)

    def str_to_bool(self, value, default):
        if isinstance(value, str):
            value = value.strip().lower()
            if value in {"true", "1", "yes", "y"}:
                return True
            elif value in {"false", "0", "no", "n"}:
                return False
        return default  # Default value if the string cannot be converted

    def get_config_boolean(self, section, key, default):
        # Allowing OTEL level overrides at top priority
        if key in self.otel_config_binding:
            otel_env_value = os.environ.get(self.otel_config_binding[key], None)
            if otel_env_value is not None and otel_env_value != "":
                return self.str_to_bool(otel_env_value, default)

        # Allowing MW level ENV overrides
        if key in self.config_binding:
            env_value = os.environ.get(self.config_binding[key], None)
            if env_value is not None and env_value != "":
                return self.str_to_bool(env_value, default)

        return self.config.getboolean(section, key, fallback=default)

    def __init__(self):
        if len(sys.argv) > 1 and sys.argv[1] == "help":
            return

        self.config_binding = {
            "project_name": "MW_PROJECT_NAME",
            "service_name": "MW_SERVICE_NAME",
            "access_token": "MW_API_KEY",
            "collect_traces": "MW_APM_COLLECT_TRACES",
            "collect_metrics": "MW_APM_COLLECT_METRICS",
            "collect_logs": "MW_APM_COLLECT_LOGS",
            "otel_propagators": "MW_PROPAGATORS",
            "mw_agent_service": "MW_AGENT_SERVICE",
            "target": "MW_TARGET",
            "custom_resource_attributes": "MW_CUSTOM_RESOURCE_ATTRIBUTES",
            "log_level": "MW_LOG_LEVEL",
            "console_exporter": "MW_CONSOLE_EXPORTER",
            "disable_info":"MW_DISABLE_INFO",
            "debug_log_file":"MW_DEBUG_LOG_FILE"
        }

        self.otel_config_binding = {
            "service_name": "OTEL_SERVICE_NAME",
            "otel_propagators": "OTEL_PROPAGATORS",
            "target": "OTEL_EXPORTER_OTLP_ENDPOINT",
            "log_level": "OTEL_LOG_LEVEL",
        }

        if not os.path.exists(CONFIG_FILE_PATH):
            print(f"Generated default config file: {CONFIG_FILE_PATH}")
            shutil.copy(DEFAULT_CONFIG_FILE_PATH, CONFIG_FILE_PATH)
        
        config_file = os.environ.get("MIDDLEWARE_CONFIG_FILE", os.path.join(os.getcwd(), 'middleware.ini'))
        self.config = configparser.ConfigParser(empty_lines_in_values=False)
        try:
            self.config.read(config_file)
        except configparser.Error as e:
            exit_with_error(f"Error reading configuration file: {e}")

        pid = os.getpid()
        self.project_name = self.get_config("middleware.common", "project_name", "")
        self.service_name = self.get_config(
            "middleware.common", "service_name", f"Service-{pid}"
        )
        self.mw_agent_service = self.get_config(
            "middleware.common", "mw_agent_service", "localhost"
        )
        self.target = self.get_config("middleware.common", "target", "")
        self.access_token = self.get_config("middleware.common", "access_token", "")
        self.collect_traces = self.get_config_boolean(
            "middleware.common", "collect_traces", True
        )
        self.collect_metrics = self.get_config_boolean(
            "middleware.common", "collect_metrics", False
        )
        self.collect_logs = self.get_config_boolean(
            "middleware.common", "collect_logs", True
        )
        self.collect_profiling = self.get_config_boolean(
            "middleware.common", "collect_profiling", False
        )
        self.otel_propagators = self.get_config(
            "middleware.common", "otel_propagators", "b3"
        )
        self.custom_resource_attributes = self.get_config(
            "middleware.common", "custom_resource_attributes", ""
        )
        self.log_level = self.get_config("middleware.common", "log_level", "INFO")
        self.console_exporter = self.get_config_boolean(
            "middleware.common", "console_exporter", False
        )
        self.debug_log_file = self.get_config_boolean(
            "middleware.common", "debug_log_file", False
        )
        self.disable_info = self.get_config_boolean(
            "middleware.common", "disable_info", False
        )

        self.resource_attributes = f"mw.app.lang=python,runtime.metrics.python=true,mw.sdk.version={SDK_VERSION},python.version={PYTHON_VERSION}"

        if self.mw_agent_service != "":
            self.exporter_otlp_endpoint = f"http://{self.mw_agent_service}:9319"
            if self.target is not None and self.target != "":
                # Add `mw_serverless` resource attribute if target is not "localhost" or agent based
                # target will have more priority over mw_agent_service
                self.resource_attributes = f"{self.resource_attributes},mw_serverless=true"
                self.exporter_otlp_endpoint = self.target

        # Passing Project name as a resource attribute
        if self.project_name is not None and self.project_name != "":
            self.resource_attributes = (
                f"{self.resource_attributes},project.name={self.project_name}"
            )

        # Passing Middleware API Key as a resource attribute, to validate ingestion requests in serverless setup
        if self.access_token is not None and self.access_token != "":
            self.resource_attributes = (
                f"{self.resource_attributes},mw.account_key={self.access_token}"
            )

        # Appending Custom Resource Attributes, if any
        if (
            self.custom_resource_attributes is not None
            and self.custom_resource_attributes != ""
        ):
            self.resource_attributes = (
                f"{self.resource_attributes},{self.custom_resource_attributes}"
            )


def exit_with_error(message):
    print(message)
    sys.exit(1)


config = Config()



import unittest
from unittest.mock import patch, mock_open
import os
import sys
from configparser import ConfigParser
from config import Config  # Replace 'your_module' with the name of your module
import os
class TestConfig(unittest.TestCase):

    def setUp(self):
        # Save the original environment variables
        self._original_env = dict(os.environ)
        
    def tearDown(self):
        # Restore the original environment variables
        os.environ.clear()
        os.environ.update(self._original_env)
        
    # If config file is empty && envs are not set, properties should be set to defaults
    def test_empty_config(self):
        # Passing test config values
        import os
        os.environ["MIDDLEWARE_CONFIG_FILE"] = os.path.join(os.getcwd(), 'testfiles/middleware_empty.ini')
        obj = Config()
        
        self.assertEqual(obj.project_name, None)
        # self.asse(obj.service_name, None)
        self.assertEqual(obj.access_token, "")
        self.assertEqual(obj.collect_traces, True)
        self.assertEqual(obj.collect_metrics, False)
        self.assertEqual(obj.collect_logs, False)
        self.assertEqual(obj.collect_profiling, False)
        self.assertEqual(obj.otel_propagators, "b3")
        self.assertEqual(obj.mw_agent_service, "localhost")
        self.assertEqual(obj.target, "")
        self.assertEqual(obj.custom_resource_attributes, "")
        self.assertEqual(obj.log_level, "FATAL")
        
    def test_env_overrides(self):
        # Setting configs with Middleware specific 
        os.environ["MIDDLEWARE_CONFIG_FILE"] = os.path.join(os.getcwd(), 'testfiles/middleware_default.ini')
        os.environ["MW_PROJECT_NAME"] = "project123"
        os.environ["MW_SERVICE_NAME"] = "service123"
        os.environ["MW_API_KEY"] = "xxxyyyzzz"
        os.environ["MW_APM_COLLECT_TRACES"] = "false"
        os.environ["MW_APM_COLLECT_METRICS"] = "true"
        os.environ["MW_APM_COLLECT_LOGS"] = "true"
        os.environ["MW_APM_COLLECT_PROFILING"] = "true"
        os.environ["MW_PROPAGATORS"] = "w3c"
        os.environ["MW_TARGET"] = "http://test.middleware.io"
        os.environ["MW_CUSTOM_RESOURCE_ATTRIBUTES"] = "test=123,test2=1234"
        os.environ["MW_LOG_LEVEL"] = "DEBUG"
    
        obj = Config()
        self.assertEqual(obj.project_name, "project123")
        self.assertEqual(obj.service_name, "service123")
        self.assertEqual(obj.access_token, "xxxyyyzzz")
        self.assertEqual(obj.collect_traces, False)
        self.assertEqual(obj.collect_metrics, True)
        self.assertEqual(obj.collect_logs, True)
        self.assertEqual(obj.collect_profiling, True)
        self.assertEqual(obj.otel_propagators, "w3c")
        self.assertEqual(obj.mw_agent_service, "localhost")
        self.assertEqual(obj.target, "http://test.middleware.io")
        self.assertEqual(obj.custom_resource_attributes, "test=123,test2=1234")
        self.assertEqual(obj.log_level, "DEBUG")
        
    def test_otel_env_overrides(self):                
        # Middleware Specific ENVs
        os.environ["MIDDLEWARE_CONFIG_FILE"] = os.path.join(os.getcwd(), 'testfiles/middleware_default.ini')
        os.environ["MW_PROJECT_NAME"] = "project123"
        os.environ["MW_SERVICE_NAME"] = "service123"
        os.environ["MW_API_KEY"] = "xxxyyyzzz"
        os.environ["MW_APM_COLLECT_TRACES"] = "false"
        os.environ["MW_APM_COLLECT_METRICS"] = "true"
        os.environ["MW_APM_COLLECT_LOGS"] = "true"
        os.environ["MW_APM_COLLECT_PROFILING"] = "true"
        os.environ["MW_PROPAGATORS"] = "w3c"
        os.environ["MW_TARGET"] = "http://test.middleware.io"
        os.environ["MW_CUSTOM_RESOURCE_ATTRIBUTES"] = "test=123,test2=1234"
        os.environ["MW_LOG_LEVEL"] = "DEBUG"
        
        # OTEL ENVs
        os.environ["OTEL_SERVICE_NAME"] = "otel-service123"
        os.environ["OTEL_PROPAGATORS"] = "otel-b3"
        os.environ["OTEL_EXPORTER_OTLP_ENDPOINT"] = "http://otel-test.middleware.io"
    
        obj = Config()
        self.assertEqual(obj.project_name, "project123")
        self.assertEqual(obj.service_name, "otel-service123")
        self.assertEqual(obj.access_token, "xxxyyyzzz")
        self.assertEqual(obj.collect_traces, False)
        self.assertEqual(obj.collect_metrics, True)
        self.assertEqual(obj.collect_logs, True)
        self.assertEqual(obj.collect_profiling, True)
        self.assertEqual(obj.otel_propagators, "otel-b3")
        self.assertEqual(obj.mw_agent_service, "localhost")
        self.assertEqual(obj.target, "http://otel-test.middleware.io")
        self.assertEqual(obj.custom_resource_attributes, "test=123,test2=1234")
        self.assertEqual(obj.log_level, "DEBUG")

    def test_str_to_bool(self):
        config = Config()
        self.assertTrue(config.str_to_bool('true', False))
        self.assertFalse(config.str_to_bool('false', True))
        self.assertTrue(config.str_to_bool('yes', False))
        self.assertFalse(config.str_to_bool('no', True))
        self.assertTrue(config.str_to_bool('1', False))
        self.assertFalse(config.str_to_bool('0', True))
        self.assertEqual(config.str_to_bool('invalid', True), True)
        self.assertEqual(config.str_to_bool('', True), True)
        
    def test_mw_serverless(self):
        
        os.environ["MW_SERVICE_NAME"] = "service123"
        os.environ["MW_TARGET"] = "http://test.middleware.io"
        obj = Config()
        self.assertIn("mw_serverless",obj.resource_attributes)
        
    def test_not_mw_serverless(self):
        os.environ["MW_SERVICE_NAME"] = "service1234"
        os.environ["MIDDLEWARE_CONFIG_FILE"] = os.path.join(os.getcwd(), 'testfiles/middleware_default.ini')
        obj = Config()
        self.assertNotIn("mw_serverless",obj.resource_attributes)
        
if __name__ == '__main__':
    unittest.main()
